package Kasei::RDFExport;

use strict;
use warnings;
use Carp qw(confess);

require Exporter;
our @ISA		= qw(Exporter);
our @EXPORT_OK	= qw(export export_leafck);

use Data::Dumper;

use RDF::Redland;
use RDF::Redland::Serializer;

our $debug		= 0;

######################################################################

our $nextns	= 0;
our %defaults	= (
				book		=> 'http://purl.org/net/schemas/book/',
				cyc			=> 'http://www.cyc.com/2004/06/04/cyc#',
				dc			=> 'http://purl.org/dc/elements/1.1/',
				dcterms		=> 'http://purl.org/dc/terms/',
				entry		=> "http://kasei.us/e/ns/mt/entry#",
				foaf		=> 'http://xmlns.com/foaf/0.1/',
				owl			=> 'http://www.w3.org/2002/07/owl#',
				rdf			=> 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
				rdfs		=> 'http://www.w3.org/2000/01/rdf-schema#',
				review		=> "http://www.purl.org/stuff/rev#",
				rss			=> "http://purl.org/rss/1.0/"
			);
our %rev_defaults	= reverse %defaults;

sub export_leafck {
	our %seen_cache;
	my $model	= shift;
	my $file	= shift;
	my $leafck	= shift;
	my @uris	= @_;
	my $fh;
	if (ref($file)) {
		$fh	= $file;
	} elsif ($file) {
		open( $fh, '>:utf8', $file ) or die $!;
	} else {
		$fh	= \*STDOUT;
	}
	my $out	= select($fh);
	print { $fh } rdf_header();

	foreach (@uris) {
		my $uri		= ref($_) ? $_ : RDF::Redland::Node->new_from_uri( $_ );
		export_rdf_node( $model, $uri, {}, 0, $leafck );
	}
	
	print { $fh } "</rdf:RDF>\n";
	select($out);
}

sub export {
	my $model	= shift;
	my $file	= shift;
	export_leafck( $model, $file, undef, @_ );
}

sub rdf_header {
	my $header	= qq(<?xml version="1.0" encoding="utf-8"?>\n)
				. qq(<rdf:RDF\n\t)
				. join("\n\t",
					map { qq(xmlns:${_}="$defaults{$_}") } (sort keys %defaults)
				)
				. "\n\t>\n";
}

sub export_rdf_node {
	my $model	= shift;
	my $node	= shift;
	my $nspaces	= shift;
	my $level	= shift;
	my $leaft	= shift;
	my $predt	= shift;
	
	warn $node->as_string if ($debug and $level <= 3);
	confess( $node->as_string ) if ($debug and $level > 10);
	
	my $indent	= "\t" x $level;
 	if ($node->is_literal) {
		my $l	= $node->literal_value;
		$l		=~ s/&/&amp;/g;
		$l		=~ s/</&lt;/g;
		$l		=~ s/"/&quot;/g;
		print $l;
	} else {
		my $leaf	= leaf_node( $model, $node, $leaft, 0 );
		my ($tag, $attr, $type, $ns)	= get_tag( $model, $node, $nspaces );
		print ($leaf ? '' : $indent);
		print tag_string($tag, $attr, $leaf);
		print ($leaf ? '' : "\n");
		
		our %seen_cache;
		$seen_cache{ $node->as_string }++;
		unless ($leaf) {
			my $st		= $model->find_statements( RDF::Redland::Statement->new( $node, undef, undef ) );
			while ($st and !$st->end) {
				my $stmt	= $st->current;
				next if (
						$stmt->predicate->equals(RDF::Redland::Node->new_from_uri("http://www.w3.org/1999/02/22-rdf-syntax-ns#type"))
					and $stmt->object->equals(RDF::Redland::Node->new_from_uri($type)));
				export_rdf_predicate( $model, $stmt->predicate, $stmt->object, $nspaces, $level + 1, $leaft, $predt );
			} continue { $st->next }
			print $indent . end_tag_string( $tag ) . "\n";
		}
		delete $nspaces->{ $ns } if ($ns);
	}
}

sub export_rdf_statement {
	my $model	= shift;
	my $st		= shift;
	my $nspaces	= shift;
	my $level	= shift;
	
	warn $st->as_string if ($debug and $level <= 3);
	
	my $node	= $st->subject;
	my $indent	= "\t" x $level;
	my ($tag, $attr, $type, $ns)	= get_tag( $model, $node, $nspaces );
	print $indent . tag_string($tag, $attr, 0) . "\n";
	
	our %seen_cache;
	$seen_cache{ $node->as_string }++;
	unless ($st->predicate->equals(RDF::Redland::Node->new_from_uri("http://www.w3.org/1999/02/22-rdf-syntax-ns#type"))
			and $st->object->equals(RDF::Redland::Node->new_from_uri($type))) {
		export_rdf_predicate( $model, $st->predicate, $st->object, $nspaces, $level + 1, sub { 1 }, sub { 1 } );
		print $indent . end_tag_string( $tag ) . "\n";
	}
	delete $nspaces->{ $ns } if ($ns);
}

sub export_rdf_predicate {
	my $model	= shift;
	my $pred	= shift;
	my $node	= shift;
	my $nspaces	= shift;
	my $level	= shift;
	my $leaft	= shift;
	my $predt	= shift || sub { 1 };
	if ($predt) {
		return unless $predt->( $pred );
	}
#	local($debug)	= 1 if ($pred->equals(RDF::Redland::Node->new_from_uri('http://xmlns.com/foaf/0.1/depicts')));
	my $indent	= "\t" x $level;
	my ($tag, $attr, $type, $ns)	= get_tag( $model, $pred, $nspaces, 1 );
	my $leaf			= leaf_node( $model, $node, $leaft, 1 );
	warn 'Predicate: ' . $pred->as_string if ($debug);
	warn 'Object   : ' . $node->as_string if ($debug);
	warn 'is leaf: [' . $leaf . ']' if ($debug);
	my $ws		= (not $leaf and not $node->is_literal and not leaf_node( $model, $node, $leaft, 0 ));
	if ($leaf and $node->is_resource) {
		$attr->{'rdf:resource'}	= $node->uri->as_string;
		print $indent . tag_string( $tag, $attr, 1 ) . "\n";
	} elsif ($leaf and $node->is_blank) {
		my $id	= $node->blank_identifier;
		if ($id =~ /^\d/) {
			$id	= '_' . $id;
		}
		$attr->{'rdf:nodeID'}	= $id;
		print $indent . tag_string( $tag, $attr, 1 ) . "\n";
	} else {
		print $indent . tag_string( $tag, $attr ) . ($ws ? "\n" : '');
		export_rdf_node( $model, $node, $nspaces, $level + 1, $leaft, $predt );
		print ($ws ? $indent : '');
		print end_tag_string( $tag ) . "\n";
	}
	delete $nspaces->{ $ns } if ($ns);
}

sub leaf_node {
	my $model	= shift;
	my $node	= shift;
	my $leaft	= shift;
	my $pred	= shift;
	if (ref($leaft)) {
		my $bool	= $leaft->( $model, $node );
		warn "-> External leaf check returned true" if ($debug and $bool);
		return 1 if ($bool);
	}
	
	our %seen_cache;
	if ($seen_cache{ $node->as_string }) {
		warn "-> This node has been seen before" if ($debug);
		return 1;
	} elsif ($pred) {
		return 0 if ($node->is_blank);
	}
	
	my $types	= 0;
	my $stmt	= RDF::Redland::Statement->new( $node, undef, undef );
	my $s	= $model->find_statements( $stmt );
	while ($s and !$s->end) {
		my $st	= $s->current;
		if ($st->predicate->equals(RDF::Redland::Node->new_from_uri("http://www.w3.org/1999/02/22-rdf-syntax-ns#type"))) {
			if ($types++ > 1) {
				warn "-> Not a leaf (more than one rdf:type)" if ($debug);
				return 0;
			}
		} else {
			warn "-> Not a leaf (more than just rdf:type statements)" if ($debug);
			return 0;
		}
	} continue { $s->next }
	warn "-> Fell through. Is not a leaf" if ($debug);
	return 1;
}

sub get_tag {
	my $model	= shift;
	my $node	= shift;
	my $nspaces	= shift;
	my $pred	= shift || 0;
	my $tag		= 'rdf:Description';
	my $attr;
	my $addedns;

	my @uris;
	if ($pred) {
		@uris	= $node->uri->as_string;
	} else {
		@uris	= get_all_uri( $model, $node, "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
		if ($node->is_resource) {
			my $url	= $node->uri->as_string;
			$attr->{'rdf:about'}	= $url;
		} elsif ($node->is_blank) {
			my $id	= $node->blank_identifier;
			if ($id =~ /^\d/) {
				$id	= '_' . $id;
			}
			$attr->{'rdf:nodeID'}	= $id;
		}
	}
	
	my $type;
	TYPE: foreach my $uri (@uris)  {
		foreach my $name (keys %{ $nspaces }) {
			my $ns	= $nspaces->{$name};
			if ((my $loc = index($uri, $ns)) == 0) {
				$tag	= join(':', $name, substr($uri,length($ns)));
				$type	= $uri;
				last TYPE;
			}
		}
	}
	
	if ($tag eq 'rdf:Description' and scalar @uris) {
		my ($ns,$t)	= $uris[0] =~ m|^(.*[#/])([^#/]+)$|;
		my $abbr	= $rev_defaults{$ns} || 'ns' . $nextns++;
		$nspaces->{$abbr}			= $ns;
		$addedns					= $abbr;
		$attr->{"xmlns:${abbr}"}	= $ns;
		$tag						= join(':', $abbr, $t);
		$type						= $uris[0];
#		die "Can't find proper namespace for predicate: $uris[0]";
#		return undef;
	}
	return ($tag, $attr, $type, $addedns);
}

sub tag_string {
	my $tag		= shift;
	my $attr	= shift;
	my $end		= shift;
	return '<' . join(' ', $tag, map { join('=', $_, map qq("$_"), $attr->{$_}) } (keys %{$attr}) ). ($end ? '/>' : '>');
}

sub end_tag_string {
	my $tag		= shift;
	return '</' . $tag . '>';
}

sub get_all_uri {
	my @nodes	= get_all_obj( @_ );
	return map { $_->uri->as_string } grep { defined($_) and $_->uri } @nodes;
}

sub get_all_obj {
	my $model	= shift;
	my $node	= shift;
	my $uri		= shift;
	my @uris	= UNIVERSAL::isa($uri, 'ARRAY') ? @{ $uri } : ($uri);
	my @preds	= map { ref($_) ? $_ : RDF::Redland::Node->new_from_uri( $_ ) } @uris;
	my @objs;
	return map { $model->targets( $node, $_ ) } @preds;
}

1;

__END__

=head1 REVISION HISTORY

 $Log: RDFExport.pm,v $
 Revision 1.4  2005/04/05 11:41:31  greg
 - removed unused modules

 Revision 1.3  2005/04/04 07:42:09  greg
 - fixed unknown namespace issue
   *** API for export_rdf_node and export_rdf_statement has changed ***
 - fixed bug with numeric nodeIDs (now prepends with '_')

 Revision 1.2  2005/04/03 23:53:36  greg
 - removed lots of namespaces
 - added code to dynamically assign namespace abbreviations.
   (this is currently broken! needs fixing)
 - added predt arg to export_rdf_node() - closure for filtering on predicates

 Revision 1.1  2005/04/03 01:44:22  greg
 - added RDF/XML export module

 Revision 1.2  2005/02/14 00:28:57  greg
 - added POD for author, license, and cvs revision history


=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
