# MT::Plugins::Redland::CMS
# -----------------
# $Revision: 1.7 $
# $Date: 2005/04/05 05:36:19 $
# ---------------------------------------------------------

package MT::Plugins::Redland::CMS;

use strict;
use warnings;

use Carp qw(confess);
use IO::All;
use Data::Dumper;
use Data::JavaScript;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME			= "CMS Hooks";
	$VERSION		= do { my @REV = split(/\./, (qw$Revision: 1.7 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug			= 0;
	$plugin_data	= {
			name		=> "${NAME}, v${VERSION}",
			description => 'Provides hooks for adding interfaces to the Entry edit page',
			doc_link	=> 'http://kasei.us/code/mtsemweb/'
		};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
}

######################################################################

no strict 'refs';
no warnings 'once';

my $glob	= 'MT::App::CMS::l10n_filter';
my $orig	= MT::App::CMS->can('l10n_filter');
*MT::App::CMS::l10n_filter	= sub {
	my $app		= shift;
	my $content	= shift;
	my(@head, @body);
	our(%headers, %boxes);
	foreach (values %boxes) {
		push( @body, ref($_) ? $_->( $app ) : $_ );
	}
	foreach (values %headers) {
		push( @head, ref($_) ? $_->( $app ) : $_ );
	}
	my $body	= jsdump('bodyblocks', \@body);
	my $head	= join("\n", @head);
	$content	=~ s{</head>}{
${head}
<style type="text/css">
#toggle {
	margin: 0 0.6em;
	padding: 5px 0 0 0;
	font-size: 80%;
	display: none;
}

#toggle ul.toggle {
	list-style: none;
	margin: 1px 0.5em;
	padding: 0;
}

#toggle ul.toggle li {
	display: inline;
	padding: 0 2px 0 0;
	margin: 0;
}

#toggle ul.toggle li a {
	border-top: 1px solid #8FAEBE;
	border-left: 1px solid #8FAEBE;
	border-right: 1px solid #8FAEBE;
	text-decoration: none;
	margin: 0;
	padding: 3px 0.4em;
}

#toggle ul.toggle li a.toggle {
	background: #C3CFDB;
	border-bottom: 1px solid #C3CFDB;
}

#toggle ul.toggle li a.toggle-inactive {
	background: #8FAEBE;
	border-bottom: 1px solid #8FAEBE;
}
#togglebox {
	border: 1px solid #8FAEBE;
	padding: 5px;
}

.semweb-box {
	border-width: 1px;
	border-color: #8FAEBE;
	border-style: solid;
	background-color: #F5F9FF;
	border-bottom: 0px;
	padding: 15px 20px;
	}
</style>
<script type="text/javascript">
function redland_shim () {
	${body}
	var body	= document.getElementById('body-box');
	var fs		= document.getElementsByTagName('form');
	for (j = 0; j < bodyblocks.length; j++) {
		var html	= bodyblocks[ j ];
		for (i = 0; i < fs.length; i++) {
			var f	= fs[ i ];
			if (f.getAttribute("name") == 'entry_form') {
				var n	= document.createElement('div');
				n.innerHTML	= html;
				n.setAttribute('class', 'semweb-box');
				f.insertBefore(n, body);
			}
		}
	}
}

function init () {
	redland_shim();
	et_init();
}

addEvent(window, 'load', init);

</script>
</head>}smix;
	if (ref($orig)) {
		return $orig->( $app, $content );
	} else {
		return $content;
	}
};

sub install_cms_box {
	my $class	= shift;
	my $name	= shift;
	my $body	= shift;
	our %boxes;
	$boxes{ $name }	= $body;
}

sub install_cms_header {
	my $class	= shift;
	my $name	= shift;
	my $body	= shift;
	our %headers;
	$headers{ $name }	= $body;
}

######################################################################

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_cms.pl,v $
 Revision 1.7  2005/04/05 05:36:19  greg
 - during update, now uses the remove_rdf_topics_of_type method from redland_cms_topic.pl

 Revision 1.6  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.5  2005/04/03 01:05:43  greg
 - code cleanups
 - added CSS and javascript for topics to use a tabbed interface

 Revision 1.4  2005/04/02 22:51:12  greg
 - updated the doc_link MT plugin variable to point to MT-Redland
 - CMS hooks can now add multiple boxes instead of all being in the same box

 Revision 1.3  2005/04/02 19:45:59  greg
 - added CVS revision and log information


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
