# MT::Plugins::Redland::CMS::Topic::Book
# -----------------
# $Revision: 1.4 $
# $Date: 2005/04/05 05:36:19 $
# ---------------------------------------------------------

package MT::Plugins::Redland::CMS::Topic::Book;

use strict;
use warnings;
no warnings 'once';

use Carp qw(confess);
use IO::All;
use File::Spec;
use Data::Dumper;
use RDF::Redland;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME			= "Redland Book Topic";
	$VERSION		= do { my @REV = split(/\./, (qw$Revision: 1.4 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug			= 0;
	$plugin_data	= {
		name		=> "${NAME}, v${VERSION}",
		description => 'Adds a book topic to an Entry',
		doc_link	=> 'http://kasei.us/code/mtredland/'
	};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms_topic.pl' );
	require $cms;
}

######################################################################

MT::Plugins::Redland::CMS::Topic->install_topic( 'Book', sub {
	my $app		= shift;
	my $q		= $app->{query};
	my $id		= $q->param('id');
	my $type	= $q->param('_type');
	my $class	= $app->_load_driver_for($type) or return;
	my $isbn	= '';
	my $more	= '';
	if ($id) {          # object exists, we're just editing it.
		my $entry	= $class->load($id) or return $app->error($app->translate("Load failed: [_1]", $class->errstr || "(no reason given)"));
		$isbn		= $entry->isbn;
		if ($isbn) {
			my $model	= $entry->driver->model;
			my $p_title	= RDF::Redland::Node->new_from_uri( "http://purl.org/dc/elements/1.1/title" );
			my $p_isbn	= RDF::Redland::Node->new_from_uri( 'http://purl.org/net/schemas/book/isbn' );
			my $l_isbn	= RDF::Redland::Node->new_literal( $isbn );
			my ($book)	= $model->sources( $p_isbn, $l_isbn );
			my ($title)	= $model->targets( $book, $p_title );
			if ($title) {
				my $title	= $title->literal_value;
				$more	= qq( <span id="bookname">${title}</span>);
			}
		}
	}
	return qq(<p><label for="isbn">ISBN</label>: <input name="isbn" id="isbn" value="${isbn}" maxlength="13" size="13" onchange="document.getElementById('bookname').innerHTML='';" /></p>)
		. $more;
});

no warnings 'redefine';
my $orig	= MT::Entry->can('column_names');
*MT::Entry::column_names	= sub {
	my $self	= shift;
	my $names	= $orig->( $self, @_ );
	push(@{ $names }, 'isbn');
	return $names;
};

MT::Entry->add_callback("post_load", 4, $plugin_data, sub {
	my ($cb, $terms, $args, $entry) = @_;
	my $model	= $entry->driver->model;
	my $t_book	= RDF::Redland::Node->new_from_uri( 'http://purl.org/net/schemas/book/Book' );
	my $p_isbn	= RDF::Redland::Node->new_from_uri( 'http://purl.org/net/schemas/book/isbn' );
	my $topic	= $entry->rdf_topic_of_type( $t_book );
	if ($topic) {
		my ($isbn)	= $model->targets( $topic, $p_isbn );
		warn "isbn: " . $isbn->as_string if ($debug);
		if ($isbn) {
			$entry->isbn( $isbn->literal_value );
		}
	}
});

MT::Entry->add_callback("pre_save", 4, $plugin_data, sub {
	my ($cb, $entry)	= @_;
	my $model	= $entry->driver->model;
	if (my $isbn = delete $entry->{'column_values'}{'isbn'}) {
		my $node	= $entry->get_rdf_node;
		my $t_book	= RDF::Redland::Node->new_from_uri( 'http://purl.org/net/schemas/book/Book' );
		my $p_isbn	= RDF::Redland::Node->new_from_uri( 'http://purl.org/net/schemas/book/isbn' );
		my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
		my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
		my $p_title	= RDF::Redland::Node->new_from_uri( "http://purl.org/dc/elements/1.1/title" );
		my $l_isbn	= RDF::Redland::Node->new_literal( $isbn );
		
		$entry->remove_rdf_topics_of_type( $t_book );
		my ($book)	= $model->sources( $p_isbn, $l_isbn );
		unless ($book) {
			$book	= RDF::Redland::Node->new_from_blank_identifier('r' . time . 'r' . $MT::ObjectDriver::Redland::blankseq++);
			$model->add_statement( RDF::Redland::Statement->new( $book, $p_type, $t_book ) );
			$model->add_statement( RDF::Redland::Statement->new( $book, $p_isbn, $l_isbn ) );
		}
		my $st		= RDF::Redland::Statement->new( $node, $p_topic, $book );
		$model->add_statement( $st );
		
		my ($title)	= $model->targets( $book, $p_title );
		unless ($title) {
			my $isbnnu	= RDF::Redland::Node->new_from_uri( "http://isbn.nu/${isbn}" );
			my $isbnnux	= RDF::Redland::URI->new( "http://isbn.nu/${isbn}.xml" );
			my $storage	= RDF::Redland::Storage->new("hashes", "test", "new='yes',hash-type='memory'");
			my $temp	= RDF::Redland::Model->new($storage, "");
			eval {
				my $parser	= RDF::Redland::Parser->new("rss-tag-soup");
				$parser->parse_into_model($isbnnux, $isbnnux, $temp);
				my $p_desc	= RDF::Redland::Node->new_from_uri( "http://purl.org/rss/1.0/description" );
				my (@desc)	= $temp->targets( $isbnnu, $p_desc );
				foreach my $d (@desc) {
					my $desc	= $d->literal_value;
					chomp($desc);
					if ($desc) {
						my $st	= RDF::Redland::Statement->new( $book, $p_title, $d );
						$model->add_statement( $st );
						last;
					}
				}
			};
		}
	}
});


######################################################################

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_cms_topic_book.pl,v $
 Revision 1.4  2005/04/05 05:36:19  greg
 - during update, now uses the remove_rdf_topics_of_type method from redland_cms_topic.pl

 Revision 1.3  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.2  2005/04/04 07:46:37  greg
 - moved topic_of_type() and topics() into MT::Entry namespace as
   rdf_topic_of_type() and rdf_topics()

 Revision 1.1  2005/04/03 01:07:41  greg
 - added redland CMS Topic plugin
 - moved redland_cms_book.pl to redland_cms_topic_book.pl, now uses Topic API
 - added redland_cms_topic_event.pl for Event topics

 Revision 1.5  2005/04/02 22:51:12  greg
 - updated the doc_link MT plugin variable to point to MT-Redland
 - CMS hooks can now add multiple boxes instead of all being in the same box

 Revision 1.4  2005/04/02 22:39:01  greg
 - now requires redland_cms.pl to be loaded before adding cms html through Redland CMS API

 Revision 1.3  2005/04/02 19:45:59  greg
 - added CVS revision and log information


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
