# MT::Plugins::Redland::Locations
# -----------------
# $Revision: 1.3 $
# $Date: 2005/04/05 11:29:06 $
# ---------------------------------------------------------

=head1 NAME

MT::Plugins::Redland::Locations - Adds dcterms:coverage data to Entries

=cut

package MT::Plugins::Redland::Locations;

=head1 SYNOPSIS

New triples will be added to the Redland model using dcterms:coverage.

=head1 REQUIRES

 MovableType 2.6 or higher

=head1 DESCRIPTION

Blah, Blah, Blah.

=cut

use strict;
use warnings;

use IO::All;
use Data::Dumper;
use RDF::Redland;
use Data::JavaScript;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME				= "Redland Locations";
	$VERSION			= do { my @REV = split(/\./, (qw$Revision: 1.3 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug				= 0;
	$plugin_data		= {
			name		=> "${NAME}, v${VERSION}",
			description => 'Adds location data to an Entry',
			doc_link	=> 'http://kasei.us/code/mtredland/'
		};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms.pl' );
	require $cms;
}

use Data::Dumper;
use HTML::Parser;

######################################################################

our $plugin;
my $cfg		= MT::ConfigMgr->instance;
if ($cfg->ObjectDriver eq 'Redland') {
	no warnings 'redefine';
	my $orig	= MT::Entry->can('column_names');
	*MT::Entry::column_names	= sub {
		my $self	= shift;
		my $names	= $orig->( $self, @_ );
		push(@{ $names }, qw(location_lat location_long location_name location_node));
		return $names;
	};
	
	my $ns			= 'http://purl.org/dc/terms/';
	my $t_point		= RDF::Redland::Node->new_from_uri( "http://www.w3.org/2003/01/geo/wgs84_pos#Point" );
	my $p_type		= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
	my $p_coverage	= RDF::Redland::Node->new_from_uri( "http://purl.org/dc/terms/coverage" );
	my $p_spatial	= RDF::Redland::Node->new_from_uri( "http://purl.org/dc/terms/spatial" );
	my $p_lat		= RDF::Redland::Node->new_from_uri( "http://www.w3.org/2003/01/geo/wgs84_pos#lat" );
	my $p_long		= RDF::Redland::Node->new_from_uri( "http://www.w3.org/2003/01/geo/wgs84_pos#long" );
	my $p_name		= RDF::Redland::Node->new_from_uri( "http://xmlns.com/foaf/0.1/name" );
	
	MT::Plugins::Redland::CMS->install_cms_header( ${NAME}, sub {
		my $app	= shift;
	return <<"END"
<style type="text/css">
	#Location input#location_name { width: 300px; }
	#Location input#location_lat, #Location input#location_long { width: 7em; }
</style>
END
} );

	MT::Plugins::Redland::CMS->install_cms_box( $NAME, sub {
		my $app		= shift;
		my $q		= $app->{query};
		my $mode	= $q->param('__mode') || '';
		my $type	= $q->param('_type') || '';
		if ($type eq 'entry' and $mode eq 'view') {
			my $model	= MT::Entry->driver->model;
			my ($lat, $long, $name)	= ('') x 3;
			my $default	= '';
			my $id 		= $q->param('id');
			if ($id) {
				my $entry	= MT::Entry->load( $id );
				if ($entry) {
					my $pnode	= $entry->location_node;
					my $point	= parse_node( $pnode );
					if ($point) {
						my ($l_lat)		= $model->targets( $point, $p_lat );
						my ($l_long)	= $model->targets( $point, $p_long );
						my ($l_name)	= $model->targets( $point, $p_name );
						$lat			= $l_lat->literal_value if ($l_lat and $l_lat->is_literal);
						$long			= $l_long->literal_value if ($l_long and $l_long->is_literal);
						$name			= $l_name->literal_value if ($l_name and $l_name->is_literal);
						$default		= "${lat},${long}";
					}
				}
			}
			
			my @point_nodes	= ('');
			my %point_nodes	= ( '' => 'Select' );
			my (@points)	= $model->sources( $p_type, $t_point );
			foreach my $point (@points) {
				my $node		= $point->as_string;
				my ($l_lat)		= $model->targets( $point, $p_lat );
				my ($l_long)	= $model->targets( $point, $p_long );
				my ($l_name)	= $model->targets( $point, $p_name );
				my $lat			= $l_lat->literal_value if ($l_lat and $l_lat->is_literal);
				my $long		= $l_long->literal_value if ($l_long and $l_long->is_literal);
				my $name		= $l_name->literal_value if ($l_name and $l_name->is_literal);
				if ($lat and $long and $name) {
					my $value	= "${lat},${long}";
					push(@point_nodes, $value);
					$point_nodes{ $value }	= $name;
				}
			}
			
			my $popup	= $q->popup_menu(
							-name		=> 'location_node',
							-values		=> \@point_nodes,
							-labels		=> \%point_nodes,
							-default	=> $default,
							-onchange	=> 'var l=this[this.selectedIndex].value.split(","); var n=this[this.selectedIndex].text; document.getElementById("location_name").setAttribute("value", n); document.getElementById("location_lat").setAttribute("value", l[0]); document.getElementById("location_long").setAttribute("value", l[1]);'
						) . "\n";
			my $body	= qq(<div id="Location">\n)
						. qq(<div><label>Location</label></div>\n)
						. qq(<div id="existingloc" style="float: right;"><div><label>Existing Location</label></div>\n${popup}</div>\n)
						. qq(<div id="newloc">\n)
						. qq(<p><label for="location_name">Name</label>: <input name="location_name" id="location_name" value="$name" /></p>\n)
						. qq(<p><label for="location_lat">Latitude</label>: <input name="location_lat" id="location_lat" value="$lat" />\n)
						. qq(&nbsp;<label for="location_long">Longitude</label>: <input name="location_long" id="location_long" value="$long" /></p>\n)
						. qq(</div>\n)
						. qq(</div>\n);
			return $body;
		}
	} );

	MT::Entry->add_callback("post_load", 6, $plugin_data, sub {
		my ($cb, $terms, $args, $entry) = @_;
		my $model	= $entry->driver->model;
		my $node	= $entry->get_rdf_node;
		my (@cover)	= $model->targets( $node, $p_coverage );
		foreach my $coverage (@cover) {
			my $st		= RDF::Redland::Statement->new( $coverage, $p_type, $t_point );
			next unless $model->contains_statement( $st );
			my ($l_lat)		= $model->targets( $coverage, $p_lat );
			my ($l_long)	= $model->targets( $coverage, $p_long );
			my ($l_name)	= $model->targets( $coverage, $p_name );
			my ($lat, $long, $name)	= ('') x 3;
			$lat			= $l_lat->literal_value if ($l_lat and $l_lat->is_literal);
			$long			= $l_long->literal_value if ($l_long and $l_long->is_literal);
			$name			= $l_name->literal_value if ($l_name and $l_name->is_literal);
			$entry->location_lat( $lat ) if ($lat);
			$entry->location_long( $long ) if ($long);
			$entry->location_name( $name ) if ($name);
			$entry->location_node( $coverage->as_string );
			last;
		}
	});
	
	MT::Entry->add_callback("pre_save", 6, $plugin, sub {
		my ($cb, $entry)	= @_;
		my $html	= $entry->text;
		my $model	= $entry->driver->model;
		my $pnode	= delete $entry->{'column_values'}{'location_node'};
		my $lat		= delete $entry->{'column_values'}{'location_lat'};
		my $long	= delete $entry->{'column_values'}{'location_long'};
		my $name	= delete $entry->{'column_values'}{'location_name'};
		my ($node)	= $entry->get_rdf_node;
		
		my $point;
		if ($name) {
			my $rdql		= qq|SELECT ?p WHERE (?p foaf:name "$name") (?p rdf:type geo:Point) USING geo FOR <http://www.w3.org/2003/01/geo/wgs84_pos#>, foaf FOR <http://xmlns.com/foaf/0.1/>|;
			my $query		= RDF::Redland::Query->new($rdql);
			my $iter		= $model->query_execute($query);
			if ($iter and not $iter->finished) {
				$point	= $iter->binding_value(0);
			}
		}
		
		if ($pnode and not $point) {
			my ($lat, $long)	= split(',', $pnode);
			my $rdql		= qq|SELECT ?p WHERE (?p geo:lat "$lat") (?p geo:long "$long") USING geo FOR <http://www.w3.org/2003/01/geo/wgs84_pos#>|;
			my $query		= RDF::Redland::Query->new($rdql);
			my $iter		= $model->query_execute($query);
			if ($iter and not $iter->finished) {
				$point	= $iter->binding_value(0);
			}
		}
		
		unless ($point) {
			my (@cover)	= $model->targets( $node, $p_coverage );
			foreach my $coverage (@cover) {
				my $st	= RDF::Redland::Statement->new( $coverage, $p_type, $t_point );
				next unless $model->contains_statement( $st );
				$point	= $coverage;
				last;
			}
		}
		
		unless ($point) {
			$point	= RDF::Redland::Node->new_from_blank_identifier('r' . time . 'r' . $MT::ObjectDriver::Redland::blankseq++);
			$model->add_statement( RDF::Redland::Statement->new( $point, $p_type, $t_point ) );
		}
		
		# XXX should only remove coverage nodes of type geo:Point
		my $st	= RDF::Redland::Statement->new( $node, $p_coverage, undef );
		my @st	= $model->find_statements( $st );
		foreach my $st (@st) {
			next if ($st->object->equals($point));
			$model->remove_statement( $st );
		}
		$st->object( $point );
		$model->add_statement( $st );
		
		foreach ([$p_lat, $lat], [$p_long, $long], [$p_name, $name]) {
			my ($p, $l)	= @{ $_ };
			next unless ($l);
			if ($node) {
				my $st	= RDF::Redland::Statement->new( $point, $p, undef );
				my @st	= $model->find_statements( $st );
				$model->remove_statement( $_ ) for (@st);
				$st->object( RDF::Redland::Node->new_literal( $l ) );
				$model->add_statement( $st );
			}
		}
	});
}

sub parse_node {
	$_			= shift;
	if (/^\[(.*)\]$/) {
		return RDF::Redland::Node->new_from_uri( $1 );
	} elsif (/^\((.*)\)$/) {
		return RDF::Redland::Node->new_from_blank_identifier( $1 );
	} elsif (/^"(.*)"$/ or /^'(.*)'$/) {
		return RDF::Redland::Node->new( $1 );
	} else {
		warn "Can't work with data: '$_'" if ($debug);
		return undef;
	}
}

######################################################################

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_locations.pl,v $
 Revision 1.3  2005/04/05 11:29:06  greg
 - fixed crashing bug where code tried to call a RDF::Redland::Node method on undef

 Revision 1.2  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.1  2005/04/04 07:43:19  greg
 - added locations plugin for dcterms:coverage triples

 Revision 1.4  2005/04/03 23:58:17  greg
 - added CVS Log POD
 - removed "see also" pod from reviews plugin that were copied from references plugin


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
