# MT::Plugins::Redland::Reviews
# -----------------
# $Revision: 1.7 $
# $Date: 2005/04/05 11:30:48 $
# ---------------------------------------------------------

=head1 NAME

MT::Plugins::Redland::Reviews - Adds review:hasReview statements to the Redland model

=cut

package MT::Plugins::Redland::Reviews;

=head1 SYNOPSIS

New triples will be added to the Redland model giving the entry topic
a rating value from 1 to 10. The Entry new/edit page is updated to
include a popup menu for the rating.

=head1 REQUIRES

 MovableType 2.6 or higher

=head1 DESCRIPTION

Blah, Blah, Blah.

=cut

use strict;
use warnings;

use IO::All;
use Data::Dumper;
use RDF::Redland;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME				= "Redland Reviews";
	$VERSION			= do { my @REV = split(/\./, (qw$Revision: 1.7 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug				= 0;
	$plugin_data		= {
			name		=> "${NAME}, v${VERSION}",
			description => "Adds rating data to an Entry's topic",
			doc_link	=> 'http://kasei.us/code/mtredland/'
		};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms_topic.pl' );
	require $cms;
}

use Data::Dumper;
use HTML::Parser;

######################################################################

our $plugin;
my $cfg		= MT::ConfigMgr->instance;
if ($cfg->ObjectDriver eq 'Redland') {
	no warnings 'redefine';
	my $orig	= MT::Entry->can('column_names');
	*MT::Entry::column_names	= sub {
		my $self	= shift;
		my $names	= $orig->( $self, @_ );
		push(@{ $names }, 'topic_rating');
		return $names;
	};
	
	MT::Plugins::Redland::CMS->install_cms_box( $NAME, sub {
		my $app		= shift;
		my $q		= $app->{query};
		my $mode	= $q->param('__mode') || '';
		my $type	= $q->param('_type') || '';
		if ($type eq 'entry' and $mode eq 'view') {
			my $default	= '';
			my $id = $q->param('id');
			if ($id) {
				my $entry	= MT::Entry->load( $id );
				if ($entry) {
					$default	= $entry->topic_rating;
				}
			}
			my $popup	= $q->popup_menu(
							-name		=> 'topic_rating',
							-values		=> ['', 1 .. 10],
							-labels		=> { '' => 'Select' },
							-default	=> $default,
							-override	=> 1
						) . "\n";
			my $body	= qq(<div><label for="topic_rating">Topic Rating</label></div>\n)
						. $popup;
			return $body;
		}
	} );


	my $ns		= 'http://www.purl.org/stuff/rev#';
	my $p_type		= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
	my $p_has		= RDF::Redland::Node->new_from_uri( "${ns}hasReview" );
	my $t_rev		= RDF::Redland::Node->new_from_uri( "${ns}Review" );
	my $p_rating	= RDF::Redland::Node->new_from_uri( "${ns}rating" );
	my $p_reviewer	= RDF::Redland::Node->new_from_uri( "${ns}reviewer" );
	
	MT::Entry->add_callback("post_load", 6, $plugin_data, sub {
		my ($cb, $terms, $args, $entry) = @_;
		my $model	= $entry->driver->model;
		my $author	= $entry->author->get_rdf_node;
		my @topics	= $entry->rdf_topics();
		my $t_rev	= RDF::Redland::Node->new_from_uri( "${ns}Review" );
		TOPIC: foreach my $topic (@topics) {
			my (@reviews)	= $model->targets( $topic, $p_has );
			foreach my $review (@reviews) {
				my $st	= RDF::Redland::Statement->new( $review, $p_reviewer, $author );
				if ($model->contains_statement( $st )) {
					my ($l_rating)	= $model->targets( $review, $p_rating );
					if ($l_rating and $l_rating->is_literal) {
						my $rating	= $l_rating->literal_value;
						$entry->topic_rating( $rating );
					}
				}
			}
		}
	});
	
	MT::Entry->add_callback("pre_save", 6, $plugin, sub {
		my ($cb, $entry)	= @_;
		my $html	= $entry->text;
		my $model	= $entry->driver->model;
		if (my $rating = delete $entry->{'column_values'}{'topic_rating'}) {
			$rating			= RDF::Redland::Node->new_literal( $rating );
			my ($node)		= $entry->rdf_topics;
			if ($node) {
				my $author	= $entry->author->get_rdf_node;
				
				my ($rev)	= $model->targets( $node, $p_has );
				unless ($rev) {
					$rev	= RDF::Redland::Node->new_from_blank_identifier('r' . time . 'r' . $MT::ObjectDriver::Redland::blankseq++);
					$model->add_statement( RDF::Redland::Statement->new( $rev, $p_type, $t_rev ) );
					$model->add_statement( RDF::Redland::Statement->new( $node, $p_has, $rev ) );
				}
				
				my $strev	= RDF::Redland::Statement->new( $rev, $p_reviewer, undef );
				my @revs	= $model->find_statements( $strev );
				$model->remove_statement( $_ ) for (grep { not $_->object->equals( $author ) } @revs);
				
				my $strate	= RDF::Redland::Statement->new( $rev, $p_rating, undef );
				my @ratings	= $model->find_statements( $strate );
				$model->remove_statement( $_ ) for (grep { not $_->object->equals( $rating ) } @ratings);
				
				$strev->object( $author );
				$strate->object( $rating );
				$model->add_statement( $strev );
				$model->add_statement( $strate );
			}
		}
	});
}


######################################################################

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_reviews.pl,v $
 Revision 1.7  2005/04/05 11:30:48  greg
 - fixed: code wasn't properly removing old review data

 Revision 1.6  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.5  2005/04/04 07:50:14  greg
 - fixed POD errors referring to MT::Plugins::Redland::References
 - now requires redland_cms_topic.pl
 - added CMS UI for adding ratings to an entry topic
 - now takes rating from CMS UI popup menu instead of text scraping

 Revision 1.4  2005/04/03 23:58:17  greg
 - added CVS Log POD
 - removed "see also" pod from reviews plugin that were copied from references plugin


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
