# MT::Plugins::Redland::Tags
# -----------------
# $Revision: 1.5 $
# $Date: 2005/04/04 10:12:45 $
# ---------------------------------------------------------

package MT::Plugins::Redland::Tags;

use strict;
use warnings;

use Data::Dumper;
use RDF::Redland;
use Kasei::RDFExport;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME				= "Redland Tags";
	$VERSION			= do { my @REV = split(/\./, (qw$Revision: 1.5 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug				= 0;
	our $plugin_data	= {
			name		=> "${NAME}, v${VERSION}",
			description => 'Adds template tags to export RDF from Redland',
			doc_link	=> 'http://kasei.us/code/mtredland/'
		};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
}

######################################################################

my $predt	= sub {
	my $pred	= shift;
	return 1 unless ($pred->is_resource);
	return 0 if ($pred->uri->as_string =~ m#^http://kasei.us/e/ns/mt#);
	return 1;
};

MT::Template::Context->add_tag( EntryRDF => sub {
	my $warnings;
	local($SIG{__WARN__})	= sub { $warnings .= $_[0] };
	my $ctx		= shift;
	my $args	= shift;
	my $entry	= $ctx->stash('entry') || $ctx->stash('entry_preview');
	my $model	= $entry->driver->model;
	my $node	= $entry->get_rdf_node;

	if ($node) {
		my $rdf;
		open(my $fh, '>', \$rdf);
		my $out	= select($fh);
		Kasei::RDFExport::export_rdf_node( $model, $node, {}, 0, sub { 0 }, $predt );
		select($out);
		return $rdf . "\n<!--\n$warnings\n-->\n" if ($debug);
		return $rdf;
	} else {
		return "\n<!--\n$warnings\n-->\n" if ($debug);
		return '';
	}
} );

######################################################################
1;

__END__

=head1 REVISION HISTORY

 $Log: redland_tags.pl,v $
 Revision 1.5  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.4  2005/04/04 07:47:17  greg
 - updated to use new RDFExport API

 Revision 1.3  2005/04/03 23:58:17  greg
 - added CVS Log POD
 - removed "see also" pod from reviews plugin that were copied from references plugin


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
